@@ -2,25 +2,41 @@
 #define dplyr_dplyr_H
 
 #include <Rcpp.h>
+#include <solaris/solaris.h>
+
 using namespace Rcpp ;
 
 // borrowed from Rcpp11
-#define RCPP_DEBUG_OBJECT(OBJ) Rf_PrintValue( Rf_eval( Rf_lang2( Rf_install( "str"), OBJ ), R_GlobalEnv ) ) ;    
-#define RCPP_INSPECT_OBJECT(OBJ) Rf_PrintValue( Rf_eval( Rf_lang2( Rf_install( ".Internal"), Rf_lang2( Rf_install( "inspect" ), OBJ ) ), R_GlobalEnv ) ) ;
+#ifndef RCPP_DEBUG_OBJECT
+    #define RCPP_DEBUG_OBJECT(OBJ) Rf_PrintValue( Rf_eval( Rf_lang2( Rf_install( "str"), OBJ ), R_GlobalEnv ) ) ;    
+#endif
+
+#ifndef RCPP_INSPECT_OBJECT
+    #define RCPP_INSPECT_OBJECT(OBJ) Rf_PrintValue( Rf_eval( Rf_lang2( Rf_install( ".Internal"), Rf_lang2( Rf_install( "inspect" ), OBJ ) ), R_GlobalEnv ) ) ;
+#endif
 
 #include <boost/scoped_ptr.hpp>
 #include <boost/functional/hash.hpp>
 
-#if defined(_WIN32)
-    #define dplyr_hash_map RCPP_UNORDERED_MAP
-    #define dplyr_hash_set RCPP_UNORDERED_SET
-#else
-    #include <boost/unordered_map.hpp>
-    #include <boost/unordered_set.hpp>
-    #define dplyr_hash_map boost::unordered_map
-    #define dplyr_hash_set boost::unordered_set
+#ifndef dplyr_hash_map
+    #if defined(_WIN32)
+        #define dplyr_hash_map RCPP_UNORDERED_MAP
+    #else
+        #include <boost/unordered_map.hpp>
+        #define dplyr_hash_map boost::unordered_map
+    #endif
 #endif
 
+#ifndef dplyr_hash_set
+    #if defined(_WIN32)
+        #define dplyr_hash_set RCPP_UNORDERED_SET
+    #else
+        #include <boost/unordered_set.hpp>
+        #define dplyr_hash_set boost::unordered_set
+    #endif
+#endif
+
+
 #include <tools/tools.h>
 
 namespace dplyr {
@@ -31,6 +47,7 @@ namespace dplyr {
     class DataFrameJoinVisitors ;
     class LazySubsets ;
     template <typename OUT, int INPUT_RTYPE> class Reducer ; 
+    const char* get_single_class(SEXP x) ;
 }
 dplyr::Result* get_handler( SEXP, const dplyr::LazySubsets&, const Environment& ) ;
 bool can_simplify(SEXP) ;
@@ -38,6 +55,10 @@ bool can_simplify(SEXP) ;
 inline SEXP as_symbol(SEXP x) {
     return Rf_install( CHAR(x) );
 }
+inline SEXP shared_SEXP(SEXP x){
+    SET_NAMED(x, 2 );  
+    return x ;  
+}
 void check_supported_type(SEXP) ;
 SEXP pairlist_shallow_copy(SEXP) ;
 void copy_attributes(SEXP, SEXP) ;
@@ -60,6 +81,7 @@ typedef dplyr::Result* (*HybridHandler)(SEXP, const dplyr::LazySubsets&, int) ;
 // [[Rcpp::register]]
 void registerHybridHandler( const char* , HybridHandler ) ;
 
+#include <dplyr/white_list.h>
 #include <dplyr/check_supported_type.h>
 #include <dplyr/visitor_set/visitor_set.h>
 #include <dplyr/DataFrameVisitorsIndexSet.h>
